sched: Remove register_task().
[paraslash.git] / sched.h
diff --git a/sched.h b/sched.h
index f15e4efb666738d5f12da542e3ae10c3d774132e..0b7df7e8fd72f9d90f531bcec37ddb4922f72fb7 100644 (file)
--- a/sched.h
+++ b/sched.h
 /**
  * Paraslash's scheduler.
  *
* Designed with KISS in mind. It manages two lists of tasks.  The pre_select
* list contains pointers to functions that are called before calling select()
* from the main loop. Similarly, \a post_select_list is a list of function
* pointers each of which is called after the select call. Tasks add hooks to
* these lists by registering themselves to the scheduler.
  */
 struct sched {
        /** Initial value before any pre_select call. */
@@ -29,46 +29,64 @@ struct sched {
        int max_fileno;
        /** If non-NULL, use this function instead of para_select. */
        int (*select_function)(int, fd_set *, fd_set *, struct timeval *);
-       /** Currently active pre_select functions. */
-       struct list_head pre_select_list;
-       /** Currently active post_select functions. */
-       struct list_head post_select_list;
+       /** Tasks which have been registered to the scheduler. */
+       struct list_head task_list;
 };
 
 /**
  * Paraslash's task structure.
  *
- * Before registering a task to the scheduler, the task structure must be
- * filled in properly by the caller.
+ * This is considered an internal structure and will eventually be made private.
  *
  * \sa \ref sched.
  */
 struct task {
+       /** Copied from the task_info struct during task_register(). */
+       void (*pre_select)(struct sched *s, struct task *t);
+       /** Copied from the task_info struct during task_register(). */
+       int (*post_select)(struct sched *s, struct task *t);
+       /** Whether this task is active (>=0) or in error state (<0). */
+       int error;
+       /** Position of the task in the task list of the scheduler. */
+       struct list_head node;
+       /** The task name supplied when the task was registered(). */
+       char status[255];
+       /** If less than zero, the task was notified by another task. */
+       int notification;
+       /** Whether the task structure should be freed in sched_shutdown(). */
+       bool owned_by_sched;
+       /** True if task is in error state and exit status has been queried. */
+       bool dead;
+       /** Usually a pointer to the struct containing this task. */
+       void *context;
+};
+
+/** Information that must be supplied by callers of \ref task_register(). */
+struct task_info {
+       /** Used for log messages and by \ref get_task_list(). */
+       const char *name;
        /**
-        * The pre select hook of \a t.
+        * The optional pre select method.
         *
         * Its purpose is to add file descriptors to the fd sets of the
         * scheduler and to decrease the select timeout if necessary.
         */
        void (*pre_select)(struct sched *s, struct task *t);
        /**
-        * The post select hook of \a t.
+        * The mandatory post select method.
         *
         * Its purpose is to evaluate and act upon the results of the previous
         * select call. If this function returns a negative value, the
         * scheduler unregisters the task.
         */
        int (*post_select)(struct sched *s, struct task *t);
-       /** Whether this task is in error state. */
-       int error;
-       /** Position of the task in the pre_select list of the scheduler. */
-       struct list_head pre_select_node;
-       /** Position of the task in the post_select list of the scheduler. */
-       struct list_head post_select_node;
-       /** Descriptive text and current status of the task. */
-       char status[255];
-       /** If less than zero, the task was notified by another task. */
-       int notification;
+       /**
+        * This pointer is saved when the task is register(ed). It may be
+        * queried from ->pre_select() and ->post_select() via \ref
+        * task_context(). Usually this is a pointer to the struct owned by the
+        * caller which contains the task pointer as one member.
+        */
+       void *context;
 };
 
 /**
@@ -79,12 +97,15 @@ struct task {
  */
 extern struct timeval *now;
 
-void register_task(struct sched *s, struct task *t);
+struct task *task_register(struct task_info *info, struct sched *s);
+void *task_context(struct task *t);
 int schedule(struct sched *s);
+void sched_shutdown(struct sched *s);
 char *get_task_list(struct sched *s);
 void task_notify(struct task *t, int err);
 void task_notify_all(struct sched *s, int err);
-int task_get_notification(struct task *t);
+int task_get_notification(const struct task *t);
+int task_reap(struct task **tptr);
 void sched_min_delay(struct sched *s);
 void sched_request_timeout(struct timeval *to, struct sched *s);
 void sched_request_timeout_ms(long unsigned ms, struct sched *s);
@@
  * Paraslash's scheduler.
  *
- * Designed with KISS in mind. It manages two lists of tasks.  The pre_select
- * list contains pointers to functions that are called before calling select()
- * from the main loop. Similarly, \a post_select_list is a list of function
- * pointers each of which is called after the select call. Tasks add hooks to
- * these lists by registering themselves to the scheduler.
+ * Designed with KISS in mind. It maintains a list of task structures which is
+ * extended when a new task is registered. Each task may define a pre_select
+ * function which is called from the scheduler main loop before it calls
+ * select(). Similarly, each task must define a post_select function which is
+ * called after the select call.
  */
 struct sched {